from connections import WebSocket
from tools import Message, Tool
from maker import Maker
from random import randint
from asyncio import run
from time import time

class Bot:

    def __init__(self, auth):
        self.auth = auth
        self.websocket = WebSocket(auth).handler
        self.make = Maker(auth)
        del auth

    def on_message(self, message):
        async def runner():
            async for i in self.websocket():
                await message(Message(i))
        run(runner())

    async def send_text(self, guid, text, message_id = None, mention_guid = None):
        data = {
            "object_guid": guid,
            "rnd": str(randint(10000000, 999999999)),
            "text": text.strip(),
            "reply_to_message_id": message_id
        }

        for i in ["**", "__", "``", "@@"]:
            if i in text:
                metadata = Tool.text_parser(text, mention_guid)
                data["metadata"] = {"meta_data_parts": metadata[0]}
                data["text"] = metadata[1].strip()
                break

        return await self.make.method("sendMessage", data)

    async def edit_message(self, guid, new_text, message_id, mention_guid = None):
        data = {
            "object_guid": guid,
            "text": new_text.strip(),
            "message_id": message_id,
        }

        for i in ["**", "__", "``", "@@"]:
            if i in new_text:
                metadata = Tool.text_parser(new_text, mention_guid)
                data['metadata'] = {'meta_data_parts': metadata[0]}
                data['text'] = metadata[1].strip()
                break

            return await self.make.method("editMessage", data)

    async def delete_message(self, guid, message_ids = [], type = "Global"):
        return await self.make.method(
            "deleteMessages",
            {
                "object_guid": guid,
                "message_ids": message_ids,
                "type": type
            }
        )

    async def forward_message(self, from_guid, message_ids, to_guid):
        return await self.make.method(
            "forwardMessages",
            {
                "from_object_guid": from_guid,
                "message_ids": message_ids,
                "rnd": f"{randint(100000,999999999)}",
                "to_object_guid": to_guid
            }
        )

    async def pin_message(self, guid, message_id):
        return await self.make.method(
            "setPinMessage",
            {
                "action": "Pin",
                "message_id": message_id,
                "object_guid": guid
            }
        )

    async def unpin_message(self, guid, message_id):
        return await self.make.method(
            "setPinMessage",
            {
                "action": "Unpin",
                "message_id": message_id,
                "object_guid": guid
            }
        )

    async def seen_message(self, seen_list = {}):
        """
            example:
                {"guid": message_id}
        """
        return await self.make.method("seenChats",{'seen_list': seen_list})

    async def get_messages_info(self, guid, messages_id):
        return await self.make.method(
            "getMessagesByID",
            {
                "object_guid": guid,
                "message_ids": messages_id
            }
        )

    async def get_messages_interval(self, guid, middle_message_id):
        return await self.make.method(
            "getMessagesInterval",
            {
                "object_guid": guid,
                "middle_message_id": middle_message_id
            }
        )

    async def get_chats(self, start_id = None):
        return await self.make.method("getChats", {'start_id': start_id})

    async def get_chats_update(self):
        return (await self.make.method("getChatsUpdate", {"state": str(round(time()) - 250)})).get('chats')

    async def get_chat_update(self, guid):
        return (await self.make.method(
            "getMessagesUpdates",
            {
                "object_guid": guid,
                "state": str(round(time()) - 200)
            }
        )).get('updated_messages')

    async def get_chat_info(self, guid):
        if guid.startswith("u"): data = ["getUserInfo", "user_guid"]
        elif guid.startswith("g"): data = ["getGroupInfo", "group_guid"]
        elif guid.startswith("c"): data = ["getChannelInfo", "channel_guid"]
        elif guid.startswith("b"): data = ["getBotInfo", "bot_guid"]
        elif guid.startswith("s"): data = ["getServiceInfo", "service_guid"]
        return await self.make.method(data[0],{data[1]: guid})

    async def get_chat_info_by_username(self, username):
        return await self.make.method(
            "getObjectByUsername",
            {"username": username.replace("@", "") if "@" in username else username}
        )

    async def delete_chat(self, guid, last_deleted_message_id):
        return await self.make.method(
            "deleteUserChat",
            {
                'last_deleted_message_id': last_deleted_message_id,
                'user_guid': guid
            }
        )

    async def delete_chat_history(self, guid, last_message_id):
        return await self.make.method(
            "deleteChatHistory",
            {
                "object_guid": guid, "last_message_id": last_message_id
            }
        )

    async def seen_channel_messages(self, guid, max_id, min_id):
        return await self.make.method(
            "seenChannelMessages",
            {
                "channel_guid": guid,
                "max_id": int(max_id),
                "min_id": int(min_id)
            }
        )

    async def send_chat_activity(self, guid, action):
        return await self.make.method(
            "sendChatActivity",
            {
                "activity": action, "object_guid": guid
            }
        )

    async def make_channel(self, title, type = "Private", member_guids = []):
        return await self.make.method(
            "addChannel",
            {
                "channel_type": type,
                "title": title,
                "member_guids": member_guids
            }
        )

    async def delete_channel(self, guid):
        return await self.make.method("removeChannel", {"channel_guid": guid})

    async def make_group(self, title, member_guids = []):
        return await self.make.method(
            "addGroup",
            {
                "title": title,
                "member_guids": member_guids
            }
        )

    async def join_channel(self, guid):
        return await self.make.method(
            "joinChannelAction",
            {
                "action": "Join",
                "channel_guid": guid
            }
        )

    async def join_channel_by_link(self, link):
        return await self.make.method("joinChannelByLink", {'hash_link': link.split('/')[-1]})

    async def leave_channel(self, guid):
        return await self.make.method(
            "joinChannelAction",
            {
                "action": "Leave",
                "channel_guid": guid
            }
        )

    async def join_group(self, link):
        return await self.make.method("joinGroup", {'hash_link': link.split('/')[-1]})

    async def leave_group(self, guid):
        return await self.make.method("leaveGroup", {"group_guid": guid})

    async def get_channel_preview(self, link):
        return await self.make.method("channelPreviewByJoinLink", {'hash_link': link.split('/')[-1]})

    async def get_group_preview(self, link):
        return await self.make.method("groupPreviewByJoinLink", {'hash_link': link.split('/')[-1]})

    async def get_group_online_member(self, guid):
        return await self.make.method("getGroupOnlineCount", {"group_guid": guid})

    async def get_group_admins(self, guid, get_guids_only = False):
        data = await self.make.method("getGroupAdminMembers",{"group_guid": guid})
        return [i["member_guid"] for i in data["data"]["in_chat_members"]] if get_guids_only else data

    async def set_group_access(self, guid, access):
        return await self.make.method(
            "setGroupDefaultAccess",
            {
                'access_list': access,
                'group_guid': guid
            }
        )

    async def get_channel_members(self, guid, search_text = None, start_id = None):
        return await self.make.method(
            "getChannelAllMembers",
            {
                "channel_guid": guid,
                "search_text": search_text,
                "start_id": start_id
            }
        )

    async def get_group_members(self, guid, start_id = None):
        return await self.make.method(
            "getGroupAllMembers",
            {
                "group_guid": guid,
                "start_id": start_id
            }
        )

    async def add_members_in_channel(self, guid, user_guids):
        return await self.make.method(
            "addChannelMembers",
            {
				"channel_guid": guid,
                "member_guids": user_guids
			}
        )

    async def add_members_in_group(self, guid, user_guids):
        return await self.make.method(
            "addGroupMembers",
            {
				"group_guid": guid,
				"member_guids": user_guids
			}
        )

    async def ban_channel_member(self, guid, user_guid):
        return await self.make.method(
            "banChannelMember",
            {
                "channel_guid": guid,
                "member_guid": user_guid,
                "action": "Set"
            }
        )

    async def ban_group_member(self, guid, user_guid):
        return await self.make.method(
            "banGroupMember",
            {
                "group_guid": guid,
                "member_guid": user_guid,
                "action": "Set"
            }
        )

    async def unban_channel_member(self, guid, user_guid):
        return await self.make.method(
            "banChannelMember",
            {
                "channel_guid": guid,
                "member_guid": user_guid,
                "action": "Unset"
            }
        )

    async def unban_group_member(self, guid, user_guid):
        return await self.make.method(
            "banGroupMember",
            {
                "group_guid": guid,
                "member_guid": user_guid,
                "action": "Unset"
            }
        )

    async def get_banned_group_members(self, guid):
        return await self.make.method("getBannedGroupMembers", {"group_guid": guid})

    async def set_channel_link(self, guid):
        return await self.make.method("setChannelLink",{"channel_guid": guid})

    async def set_group_link(self, guid):
        return await self.make.method("setGroupLink", {"group_guid": guid})

    async def get_channel_link(self, guid):
        return await self.make.method("getChannelLink",{"channel_guid": guid})

    async def get_group_link(self, guid):
        return await self.make.method("getGroupLink", {"group_guid": guid})

    async def get_group_mention(self, guid):
        return await self.make.method("getGroupMentionList", {"group_guid": guid})

    async def set_group_timer(self, guid, time):
        return await self.make.method(
            "editGroupInfo",
            {
                "group_guid": guid,
                "slow_mode": time,
                "updated_parameters": ["slow_mode"]
            }
        )

    async def set_group_admin(self, guid, user_guid, access_list):
        return await self.make.method(
            "setGroupAdmin",
            {
                "group_guid": guid,
                "access_list": access_list,
                "action": "SetAdmin",
                "member_guid": user_guid
            }
        )

    async def unset_group_admin(self, guid, user_guid):
        return await self.make.method(
            "setGroupAdmin",
            {
                "group_guid": guid,
                "action": "UnsetAdmin",
                "member_guid": user_guid
            }
        )

    async def search_chats(self, title):
        return await self.make.method("searchGlobalObjects", {"search_text": title})

    async def search_messages(self, text, type, start_id = 0):
        return await self.make.method(
            "searchGlobalMessages",
            {
                "search_text":text,
                "type": type,
                "start_id": start_id
            }
        )

    async def get_post_info(self, link):
        return await self.make.method("getLinkFromAppUrl", {"app_url": link})

    async def block_user(self, guid):
        return await self.make.method(
            "setBlockUser",
            {
                "action": "Block",
                "user_guid": guid
            }
        )

    async def unblock_user(self, guid):
        return await self.make.method(
            "setBlockUser",
            {
                "action": "Unblock",
                "user_guid": guid
            }
        )

    async def mute_chat(self, guid):
        return await self.make.method(
            "setActionChat",
            {
                "action": "Mute",
                "object_guid": guid
            }
        )

    async def unmute_chat(self, guid):
        return await self.make.method(
            "setActionChat",
            {
                "action": "Unmute",
                "object_guid": guid
            }
        )

    async def add_contact(self, first_name, last_name, phone_number):
        return await self.make.method(
            "addAddressBook",
            {
                "first_name": first_name,
                "last_name": last_name,
                "phone": phone_number
            }
        )

    async def change_password(self, old_password, new_password, hint):
        return await self.make.method(
            "changePassword",
            {
                "new_hint": hint,
                "new_password": new_password,
                "password": old_password
            }
        )

    async def delete_two_step(self, password):
        return await self.make.method("turnOffTwoStep", {"password": password})

    async def change_phone_number(self, new_phone_number):
        return await self.make.method(
            "requestChangePhoneNumber",
            {
                "new_phone_number": new_phone_number if new_phone_number.startswith("98") else
                "98" + new_phone_number[1:]
            }
        )

    async def get_sessions(self):
        return await self.make.method("getMySessions", {})

    async def get_sticker_sets(self):
        return await self.make.method('getMyStickerSets', {})

    async def send_poll(
        self,
        guid,
        question = "Hello from Pyrubi Library !",
        options = ["هیچ گزینه ای وارد نشده است !", "Rubika : @Persian_PyThon"],
        multiple_answers = False,
        anonymous = True,
        type = "Regular", # Another type is "Quiz"
        message_id = 0):
        return await self.make.method(
            "createPoll",
            {
                "allows_multiple_answers": multiple_answers,
                "is_anonymous": anonymous,
                "object_guid": guid,
                "options": options,
                "question": question,
                "reply_to_message_id": int(message_id),
                "rnd": randint(100000000, 999999999),
                "type": type
            }
        )

    async def edit_profile(self, **kwargs):
        if "username" in list(kwargs.keys()):
            return await self.make.method(
                "updateUsername",
                {
                    'username': kwargs.get("username"),
                    'updated_parameters': ['username']
                }
            )
        else:
            return await self.make.method(
                "updateProfile",
                {
                    "first_name": kwargs.get('first_name'),
                    "last_name": kwargs.get('last_name'),
                    "bio": kwargs.get('bio'),
                    "updated_parameters": list(kwargs.keys())
                }
            )

    async def addFolder(
        self,
        name,
        exclude_chat_types = [],
        exclude_object_guids = [],
        include_chat_types = [],
        include_object_guids = [],
        is_add_to_top = True,
        folder_id = ""
    ):
        return await self.make.method(
            "addFolder",
            dict(
                exclude_object_guids = exclude_object_guids,
                include_object_guids = include_object_guids,
                exclude_chat_types = exclude_chat_types,
                include_chat_types = include_chat_types,
                folder_id = folder_id, is_add_to_top = is_add_to_top,
                name = name
            )
            
        )